<?php

namespace Drupal\Tests\privatemsg\Functional;

use Drupal\Core\Language\LanguageInterface;
use Drupal\filter\Entity\FilterFormat;
use Drupal\taxonomy\Entity\Term;
use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;

/**
 * Tests for the privatemsg module.
 *
 * @group privatemsg
 */
class PrivatemsgModeratorComplainTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to install.
   *
   * @var array
   */
  public static $modules = [
    'privatemsg',
  ];

  /**
   * The User used for the test.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  private $user1;

  /**
   * The User used for the test.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  private $user2;

  /**
   * The User used for the test.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  private $userModerator;

  /**
   * Role for moderator.
   *
   * @var \Drupal\user\Entity\Role
   */
  protected $role;

  /**
   * Term for referencing.
   *
   * @var \Drupal\taxonomy\TermInterface
   */
  protected $term;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create Full HTML text format.
    $full_html_format = FilterFormat::create([
      'format' => 'full_html',
      'name' => 'Full HTML',
    ]);
    $full_html_format->save();

    $this->user1 = $this->DrupalCreateUser([
      'privatemsg write messages',
      $full_html_format->getPermissionName(),
    ]);

    $this->user2 = $this->DrupalCreateUser([
      'privatemsg write messages',
      $full_html_format->getPermissionName(),
    ]);

    $this->role = Role::create([
      'id' => 'moderator',
      'label' => 'Moderator',
    ]);
    $this->role->save();
    $config = \Drupal::configFactory()->getEditable('privatemsg.settings');
    $config->set('moderator_role', 'moderator');
    $config->save();

    $this->userModerator = $this->DrupalCreateUser([
      'privatemsg write messages',
    ]);
    $this->userModerator->addRole('moderator');
    $this->userModerator->save();

    $this->term = Term::create([
      'name' => 'Spam',
      'vid' => 'privatemsg_claim_reason',
      'langcode' => LanguageInterface::LANGCODE_NOT_SPECIFIED,
    ]);
    $this->term->save();
  }

  /**
   * Tests invite moderator to the thread.
   */
  public function testInviteModerator() {
    $this->drupalLogin($this->user1);
    $this->drupalGet('/messages/new');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'thread_subject' => 'Invite moderator test',
      'thread_members' => $this->user2->getDisplayName() . ' (' . $this->user2->id() . ')',
      'message[0][value]' => $this->getRandomGenerator()->sentences(5),
    ], 'Send message', 'privatemsg-message-add-form');

    $this->drupalLogin($this->user2);
    $this->drupalGet('/messages/view/1');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/messages/complain/1/1');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'claim_reason' => $this->term->id(),
      'message' => $this->getRandomGenerator()->sentences(5),
    ], 'Send', 'privatemsg-complain-form');
    $this->assertSession()->pageTextContains('The message has been sent to moderator.');

    $this->drupalLogin($this->userModerator);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/messages');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextContains('css', 'td.views-field-subject a', 'Claim to moderator');
    $this->drupalGet('/messages/view/3');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Reason');
    $this->assertSession()->pageTextContains('Message from user');
    $this->assertSession()->pageTextContains('Original text');
    $this->assertSession()->pageTextContains('Url to thread');
  }

}
